@@ -1,6 +1,6 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
-//@line 38 "/builds/tinderbox/XR-Trunk/Linux_2.6.18-8.el5_Depend/mozilla/toolkit/mozapps/extensions/src/nsAddonRepository.js"
+//@line 38 "/builds/slave/mozilla-1.9.1-linux-xulrunner/build/toolkit/mozapps/extensions/src/nsAddonRepository.js"
 */
 
 const Cc = Components.classes;
@@ -17,7 +17,7 @@ const PREF_GETADDONS_GETSEARCHRESULTS    = "extensions.getAddons.search.url";
 
 const XMLURI_PARSE_ERROR  = "http://www.mozilla.org/newlayout/xml/parsererror.xml";
 
-const API_VERSION = "1";
+const API_VERSION = "1.2";
 
 function AddonSearchResult() {
 }
@@ -45,7 +45,7 @@ function AddonRepository() {
 
 AddonRepository.prototype = {
   // The current set of results
-  _addons: [],
+  _addons: null,
 
   // Whether we are currently searching or not
   _searching: false,
@@ -96,6 +96,8 @@ AddonRepository.prototype = {
       this._request.abort();
       this._request = null;
     }
+    this._callback = null;
+    this._addons = null;
   },
 
   retrieveRecommendedAddons: function(aMaxResults, aCallback) {
@@ -136,7 +138,8 @@ AddonRepository.prototype = {
 
     var uri = prefs.getCharPref(PREF_GETADDONS_GETSEARCHRESULTS);
     uri = uri.replace(/%API_VERSION%/g, API_VERSION);
-    uri = uri.replace(/%TERMS%/g, encodeURIComponent(aSearchTerms));
+    // We double encode due to bug 427155
+    uri = uri.replace(/%TERMS%/g, encodeURIComponent(encodeURIComponent(aSearchTerms)));
     uri = urlf.formatURL(uri);
     this._loadList(uri);
   },
@@ -145,15 +148,23 @@ AddonRepository.prototype = {
   _reportSuccess: function(aCount) {
     this._searching = false;
     this._request = null;
-    this._callback.searchSucceeded(this._addons, this._addons.length,
-                                   this._recommended ? -1 : aCount);
+    // The callback may want to trigger a new search so clear references early
+    var addons = this._addons;
+    var callback = this._callback;
+    this._callback = null;
+    this._addons = null;
+    callback.searchSucceeded(addons, addons.length, this._recommended ? -1 : aCount);
   },
 
   // Notifies the callback of a failure
   _reportFailure: function(aEvent) {
     this._searching = false;
     this._request = null;
-    this._callback.searchFailed();
+    // The callback may want to trigger a new search so clear references early
+    var callback = this._callback;
+    this._callback = null;
+    this._addons = null;
+    callback.searchFailed();
   },
 
   // Parses an add-on entry from an <addon> element
@@ -208,10 +219,10 @@ AddonRepository.prototype = {
       return;
     var vc = Cc["@mozilla.org/xpcom/version-comparator;1"].
              getService(Ci.nsIVersionComparator);
-    var apps = tags[0].getElementsByTagName("name");
+    var apps = tags[0].getElementsByTagName("appID");
     var i = 0;
     while (i < apps.length) {
-      if (apps[i].textContent.toLowerCase() == app.name.toLowerCase()) {
+      if (apps[i].textContent == app.ID) {
         var minversion = apps[i].parentNode.getElementsByTagName("min_version")[0].textContent;
         var maxversion = apps[i].parentNode.getElementsByTagName("max_version")[0].textContent;
         if ((vc.compare(minversion, app.version) > 0) ||
